# -*- coding: utf-8 -*-

import logging
import os

import inventree.base

logger = logging.getLogger('inventree')


class LabelPrintingMixin:
    """Mixin class for label printing.

    Classes which implement this mixin should define the following attributes:

    LABELNAME: The name of the label type (e.g. 'part', 'stock', 'location')
    LABELITEM: The name of the label item (e.g. 'parts', 'items', 'locations')
    """

    LABELNAME = ''
    LABELITEM = ''

    def printlabel(self, label, plugin=None, destination=None, *args, **kwargs):
        """Print the label belonging to the given item.

        Set the label with 'label' argument, as the ID of the corresponding
        label. A corresponding label object can also be given.

        If a plugin is given, the plugin will determine
        how the label is printed, and a message is returned.

        Otherwise, if a destination is given, the file will be downloaded to 'destination'.
        Use overwrite=True to overwrite an existing file.

        If neither plugin nor destination is given, nothing will be done
        """

        if isinstance(label, (LabelPart, LabelStock, LabelLocation)):
            label_id = label.pk
        else:
            label_id = label

        # Set URL to use
        URL = f'/label/{self.LABELNAME}/{label_id}/print/'

        params = {
            f'{self.LABELITEM}[]': self.pk
        }

        if plugin is not None:
            # Append profile
            params['plugin'] = plugin

        # If API version less than 130, file download is provided directly
        if self._api.api_version < 130 and plugin is None:
            # Ensure we prefix the URL with '/api'
            download_url = f"/api{URL}"
        else:
            # Perform API request, get response
            response = self._api.get(URL, params=params)
            download_url = response.get('file', None)

        # Label file is available for download
        if download_url and destination is not None:
            if os.path.exists(destination) and os.path.isdir(destination):
                # No file name given, construct one
                # Otherwise, filename will be something like '?parts[]=37'
                destination = os.path.join(
                    destination,
                    f'Label_{self.LABELNAME}{label}_{self.pk}.pdf'
                )

            # Use downloadFile method to get the file
            return self._api.downloadFile(url=download_url, destination=destination, params=params, *args, **kwargs)

        else:
            return response


class LabelLocation(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """ Class representing the Label/Location database model """

    URL = 'label/location'


class LabelPart(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """ Class representing the Label/Part database model """

    URL = 'label/part'


class LabelStock(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """ Class representing the Label/stock database model """

    URL = 'label/stock'



# -*- coding: utf-8 -*-

import os
import sys

sys.path.append(os.path.abspath(os.path.dirname(__file__)))

from test_api import InvenTreeTestCase  # noqa: E402
from inventree.part import BomItem  # noqa: E402
from inventree.build import Build  # noqa: E402
from inventree.purchase_order import PurchaseOrder  # noqa: E402
from inventree.sales_order import SalesOrder  # noqa: E402
from inventree.return_order import ReturnOrder  # noqa: E402
from inventree.stock import StockLocation, StockItemTestResult  # noqa: E402
from inventree.report import (ReportBoM, ReportBuild, ReportPurchaseOrder, ReportSalesOrder, ReportReturnOrder, ReportStockLocation, ReportTest)  # noqa: E402


class ReportClassesTest(InvenTreeTestCase):
    """Tests for Report functions models"""

    def test_report_list(self):
        """
        Test for using listing functionality of each type of label
        """

        for RepClass in (ReportBoM, ReportBuild, ReportPurchaseOrder, ReportSalesOrder, ReportReturnOrder, ReportStockLocation, ReportTest):
            # Get report list
            report_list = RepClass.list(self.api)

            # Make a new list filtered by LabelPart
            # List should be non-zero length
            # Filtered list should be equal the original list
            report_list_filtered = [x for x in report_list if isinstance(x, RepClass)]

            self.assertGreater(len(report_list_filtered), 0)
            self.assertEqual(report_list, report_list_filtered)

    def test_report_printing(self):
        """
        Tests for using report printing function to download PDF files
        """

        # For each class supporting reports, find a related object
        # Define a file, write the label to this file
        # Check for file

        testscount = 0

        for ItemModel, RepClass in zip(
                (BomItem, Build, PurchaseOrder, SalesOrder, ReturnOrder, StockLocation, StockItemTestResult),
                (ReportBoM, ReportBuild, ReportPurchaseOrder, ReportSalesOrder, ReportReturnOrder, ReportStockLocation, ReportTest)
        ):

            # Item and report - get first in list
            itmlist = ItemModel.list(self.api)
            replist = RepClass.list(self.api)

            # The test can only be carried out if there are items and reports defined
            if len(itmlist) > 0 and len(replist) > 0:
                itm = itmlist[0]
                rep = replist[0]

                # Attempt to print to file - use label object
                self.assertTrue(itm.printreport(report=rep, destination="report.pdf"))

                # Make sure the files exist
                self.assertTrue(os.path.isfile("report.pdf"))

                os.remove("report.pdf")

                testscount += 1

        # Make sure we actually tested something
        assert testscount > 0



# -*- coding: utf-8 -*-

import inventree.base


class ReportPrintingMixin:
    """Mixin class for report printing.

    Classes which implement this mixin should define the following attributes:

    REPORTNAME: The name of the report type, as given in the API (e.g. 'bom', 'build', 'po')
    See https://demo.inventree.org/api-doc/#tag/report
    REPORTITEM: The name of items send to the report print endpoint.
    This is the name of the array of items expected.

    Example:
    https://demo.inventree.org/api/report/test/?item[]=205
        -> REPORTNAME = test
        -> REPORTITEM = item
    """

    REPORTNAME = ''
    REPORTITEM = ''

    def printreport(self, report, destination, *args, **kwargs):
        """Print the report belonging to the given item.

        Set the report with 'report' argument, as the ID of the corresponding
        report. A corresponding report object can also be given.

        The file will be downloaded to 'destination'.
        Use overwrite=True to overwrite an existing file.

        If neither plugin nor destination is given, nothing will be done
        """

        if isinstance(report, (ReportBoM, ReportBuild, ReportPurchaseOrder, ReportSalesOrder, ReportReturnOrder, ReportStockLocation, ReportTest)):
            report_id = report.pk
        else:
            report_id = report

        # Set URL to use
        URL = f'/api/report/{self.REPORTNAME}/{report_id}/print/'

        params = {
            f'{self.REPORTITEM}[]': self.pk
        }

        # Use downloadFile method to get the file
        return self._api.downloadFile(url=URL, destination=destination, params=params, *args, **kwargs)


class ReportBoM(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """Class representing ReportBoM"""

    URL = 'report/bom'


class ReportBuild(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """Class representing ReportBuild"""

    URL = 'report/build'


class ReportPurchaseOrder(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """Class representing ReportPurchaseOrder"""

    URL = 'report/po'


class ReportSalesOrder(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """Class representing ReportSalesOrder"""

    URL = 'report/so'


class ReportReturnOrder(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """Class representing ReportReturnOrder"""

    URL = 'report/ro'


class ReportStockLocation(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """Class representing ReportStockLocation"""

    URL = 'report/slr'


class ReportTest(inventree.base.MetadataMixin, inventree.base.InventreeObject):
    """Class representing ReportTest"""

    URL = 'report/test'
